use crate::ui::{UIWidget, color::UIColor};
use maud::{Markup, Render, html};

#[allow(non_snake_case)]
/// Text UI Widget
#[must_use]
pub fn Text(txt: &str) -> TextWidget {
    TextWidget {
        inner: None,
        txt: txt.to_string(),
        family: String::new(),
        font: String::new(),
        color: String::new(),
        style: Vec::new(),
        size: String::new(),
        span: false,
    }
}

#[allow(non_snake_case)]
/// HTML `<p>` Paragraph
pub fn Paragraph<T: UIWidget + 'static>(inner: T) -> TextWidget {
    TextWidget {
        inner: Some(Box::new(inner)),
        font: String::new(),
        family: String::new(),
        color: String::new(),
        txt: String::new(),
        style: Vec::new(),
        size: String::new(),
        span: false,
    }
}

#[allow(non_snake_case)]
/// `<span>` element
#[must_use]
pub fn Span(txt: &str) -> TextWidget {
    TextWidget {
        inner: None,
        txt: txt.to_string(),
        family: String::new(),
        font: String::new(),
        style: Vec::new(),
        color: String::new(),
        size: String::new(),
        span: true,
    }
}

pub struct TextWidget {
    inner: Option<Box<dyn UIWidget>>,
    txt: String,
    family: String,
    style: Vec<String>,
    font: String,
    color: String,
    size: String,
    span: bool,
}

impl TextWidget {
    // Weight

    #[must_use]
    pub fn thin(mut self) -> Self {
        self.font = "font-thin".to_string();
        self
    }

    #[must_use]
    pub fn extralight(mut self) -> Self {
        self.font = "font-extralight".to_string();
        self
    }

    #[must_use]
    pub fn light(mut self) -> Self {
        self.font = "font-light".to_string();
        self
    }

    #[must_use]
    pub fn normal(mut self) -> Self {
        self.font = "font-normal".to_string();
        self
    }

    /// Turn `Text` medium.
    ///
    /// Adds the class `font-medium`
    #[must_use]
    pub fn medium(mut self) -> Self {
        self.font = "font-medium".to_string();
        self
    }

    /// Turn `Text` semibold.
    ///
    /// Adds the class `font-semibold`
    #[must_use]
    pub fn semibold(mut self) -> Self {
        self.font = "font-semibold".to_string();
        self
    }

    /// Turn `Text` bold.
    ///
    /// Adds the class `font-bold`
    #[must_use]
    pub fn bold(mut self) -> Self {
        self.font = "font-bold".to_string();
        self
    }

    #[must_use]
    pub fn extrabold(mut self) -> Self {
        self.font = "font-extrabold".to_string();
        self
    }

    #[must_use]
    pub fn weight_black(mut self) -> Self {
        self.font = "font-black".to_string();
        self
    }

    // Styles

    #[must_use]
    pub fn italic(mut self, apply: bool) -> Self {
        if apply {
            self.style.push("italic".to_string());
        } else {
            self.style.push("not-italic".to_string())
        }
        self
    }

    #[must_use]
    pub fn number_style(mut self, s: NumberStyle) -> Self {
        self.style.push(s.to_value().to_string());
        self
    }

    // Sizes

    #[must_use]
    pub fn _9xl(mut self) -> Self {
        self.size = "text-9xl".to_string();
        self
    }

    #[must_use]
    pub fn _8xl(mut self) -> Self {
        self.size = "text-8xl".to_string();
        self
    }

    #[must_use]
    pub fn _7xl(mut self) -> Self {
        self.size = "text-7xl".to_string();
        self
    }

    #[must_use]
    pub fn _6xl(mut self) -> Self {
        self.size = "text-6xl".to_string();
        self
    }

    #[must_use]
    pub fn _5xl(mut self) -> Self {
        self.size = "text-5xl".to_string();
        self
    }

    #[must_use]
    pub fn _4xl(mut self) -> Self {
        self.size = "text-4xl".to_string();
        self
    }

    #[must_use]
    pub fn large(mut self) -> Self {
        self.size = "text-lg".to_string();
        self
    }

    #[must_use]
    pub fn base_size(mut self) -> Self {
        self.size = "text-base".to_string();
        self
    }

    /// Turn `Text` size to 3XL.
    ///
    /// Adds the class `text-3xl`
    #[must_use]
    pub fn _3xl(mut self) -> Self {
        self.size = "text-3xl".to_string();
        self
    }

    /// Turn `Text` size to 2XL.
    ///
    /// Adds the class `text-2xl`
    #[must_use]
    pub fn _2xl(mut self) -> Self {
        self.size = "text-2xl".to_string();
        self
    }

    /// Turn `Text` size to xl.
    ///
    /// Adds the class `text-xl`
    #[must_use]
    pub fn xl(mut self) -> Self {
        self.size = "text-xl".to_string();
        self
    }

    /// Turn `Text` size to small.
    ///
    /// Adds the class `text-sm`
    #[must_use]
    pub fn sm(mut self) -> Self {
        self.size = "text-sm".to_string();
        self
    }

    /// Turn `Text` size to x small.
    ///
    /// Adds the class `text-xs`
    #[must_use]
    pub fn xs(mut self) -> Self {
        self.size = "text-xs".to_string();
        self
    }

    // Text Color

    #[must_use]
    pub fn color<T: UIColor>(mut self, color: &T) -> Self {
        self.color = format!("text-{}", color.color_class());
        self
    }

    #[must_use]
    pub fn black(mut self) -> Self {
        self.color = "text-black".to_string();
        self
    }

    #[must_use]
    pub fn white(mut self) -> Self {
        self.color = "text-white".to_string();
        self
    }

    // Font Family

    #[must_use]
    pub fn sans(mut self) -> Self {
        self.family = "font-sans".to_string();
        self
    }

    #[must_use]
    pub fn serif(mut self) -> Self {
        self.family = "font-serif".to_string();
        self
    }

    #[must_use]
    pub fn mono(mut self) -> Self {
        self.family = "font-mono".to_string();
        self
    }
}

impl Render for TextWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for TextWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        vec![
            self.color.clone(),
            self.font.clone(),
            self.size.clone(),
            self.family.clone(),
        ]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        if let Some(inner) = &self.inner {
            c.extend_from_slice(&inner.extended_class());
        }
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if let Some(inner) = &self.inner {
            if self.span {
                html! {
                    span class=(format!("{} {}", class, self.base_class().join(" "))) { (inner) }
                }
            } else {
                html! {
                    p class=(format!("{} {}", class, self.base_class().join(" "))) { (inner) }
                }
            }
        } else if self.span {
            html! {
                span class=(format!("{} {}", class, self.base_class().join(" "))) { (self.txt) }
            }
        } else {
            html! {
                p class=(format!("{} {}", class, self.base_class().join(" "))) { (self.txt) }
            }
        }
    }
}

pub enum NumberStyle {
    Normal,
    Ordinal,
    SlashedZero,
    OldStyle,
    Lining,
    Proportional,
    Tabular,
    DiagonalFractions,
    StackedFractions,
}

impl NumberStyle {
    pub const fn to_value(&self) -> &str {
        match self {
            NumberStyle::Normal => "normal-nums",
            NumberStyle::Ordinal => "ordinal",
            NumberStyle::SlashedZero => "slashed-zero",
            NumberStyle::OldStyle => "oldstyle-nums",
            NumberStyle::Lining => "lining-nums",
            NumberStyle::Proportional => "proportional-nums",
            NumberStyle::Tabular => "tabular-nums",
            NumberStyle::DiagonalFractions => "diagonal-fractions",
            NumberStyle::StackedFractions => "stacked-fractions",
        }
    }
}
